
-- Project: 7-segment
--
-- Controls two external 7-segment displays.
--
-- !!!! this is for the Digilent board !!!!

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

entity seg_control is
  port 
   (
    clk          : in   std_logic;
    pb_debncd_1  : in   std_logic;
    pb_debncd_2  : in   std_logic;
    --
    seven_seg_1  : out  std_logic_vector(7 downto 0);
    seven_seg_2  : out  std_logic_vector(7 downto 0); --not used on this board
    en1          : out  std_logic;
    en2          : out  std_logic
   );
end entity;

architecture Behavioral of seg_control is

   signal pb_debncd_1_d1 : std_logic;
   signal pb_debncd_2_d1 : std_logic;
   signal display_select : std_logic; -- 0 = #1, 1 = #2
   signal display_cnt_1  : unsigned(3 downto 0);
   signal display_cnt_2  : unsigned(3 downto 0);
   signal display_cnt    : unsigned(3 downto 0);
   
begin

   seg_control_process : process(clk)
   begin
      if rising_edge(clk) then
         pb_debncd_1_d1 <= pb_debncd_1;
         pb_debncd_2_d1 <= pb_debncd_2;
         --
         if    (    pb_debncd_1    = '1' --Digilent PBs are high-active
                AND pb_debncd_1_d1 = '0'
               ) then 
               display_select <= '0';
               display_cnt_1  <= display_cnt_1 + 1;
         elsif (    pb_debncd_2    = '1' --Digilent PBs are high-active
                AND pb_debncd_2_d1 = '0'
               ) then 
               display_select <= '1';
               display_cnt_2  <= display_cnt_2 + 1;
         end if;
      end if;
   end process;
   
   en1 <=     display_select; --7-segment enables are low-active.
   en2 <= NOT display_select;

   --      0
   --    5   1
   --      6
   --    4   2
   --      3
   
   seg_decode_process : process(clk)
   begin
      if rising_edge(clk) then
         if (display_select = '0') then
            display_cnt <= display_cnt_1;
         else
            display_cnt <= display_cnt_2;
         end if;
         --    display segments are low-active
         case(display_cnt) is         -- 76543210
            when X"0" => seven_seg_1 <= "11000000";
            when X"1" => seven_seg_1 <= "11111001";
            when X"2" => seven_seg_1 <= "10100100";
            when X"3" => seven_seg_1 <= "10110000";
            when X"4" => seven_seg_1 <= "10011001";
            when X"5" => seven_seg_1 <= "10010010";
            when X"6" => seven_seg_1 <= "10000010";
            when X"7" => seven_seg_1 <= "11111000";
            when X"8" => seven_seg_1 <= "10000000";
            when X"9" => seven_seg_1 <= "10010000";
            when X"A" => seven_seg_1 <= "10001000";
            when X"b" => seven_seg_1 <= "10000011";
            when X"C" => seven_seg_1 <= "11000110";
            when X"d" => seven_seg_1 <= "10100001";
            when X"E" => seven_seg_1 <= "10000110";
            when X"F" => seven_seg_1 <= "10001110";
            when others => seven_seg_1 <= "11111111";
         end case;
      end if;
   end process;
   
end architecture Behavioral;

