
-- Project: fifo_game
--
-- game to catch delayed tagged samples in a sequences of numbers.

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

use work.projects_pkg.all;

entity fifo is
  port 
   (
    clk          : in  std_logic;
    start        : in  std_logic;
    prbs_in      : in  std_logic_vector(2 downto 0);
    tag_in       : in  std_logic;
    wr           : in  std_logic;
    --
    rd           : in  std_logic;
    level        : out std_logic_vector(3 downto 0);
    prbs_out     : out std_logic_vector(2 downto 0);
    tag_out      : out std_logic
   );
end entity;

architecture Behavioral of fifo is

   type array_16x4 is array(0 to 15) of std_logic_vector(3 downto 0);
   signal mem : array_16x4;
   --
   signal level_cnt   : unsigned(3 downto 0);
   signal wr_adr      : unsigned(3 downto 0);
   signal wr_adr_int  : integer;
   signal rd_adr      : unsigned(3 downto 0);
   signal rd_adr_int  : integer;
   signal tag_out_lcl : std_logic;

begin


   address_proc : process(clk)
   begin
     if rising_edge(clk) then
       if (start = '1') then
          wr_adr    <= X"4";
          rd_adr    <= X"0";
          level_cnt <= X"4";
       end if;
       --
       if (wr = '1') then
          wr_adr <= wr_adr + 1;
       end if;
       --
       if (    level_cnt > 4
           AND rd = '1'
          ) then
          rd_adr <= rd_adr + 1;
       end if;
       --
       if (    level_cnt > 4
           AND rd = '1'
          ) then
          level_cnt <= level_cnt - 1;
       elsif (wr = '1') then
          level_cnt <= level_cnt + 1;
       end if;
     end if;
   end process;

   wr_adr_int <= to_integer(unsigned(wr_adr));
   rd_adr_int <= to_integer(unsigned(rd_adr));

   memory_proc : process(clk)
   begin
     if rising_edge(clk) then
       if (wr = '1') then
         mem(wr_adr_int) <= (prbs_in & tag_in);
       end if;
       --
       prbs_out     <= mem(rd_adr_int)(3 downto 1);
       tag_out_lcl  <= mem(rd_adr_int)(0);
     end if;
   end process;
   
   level   <= std_logic_vector(level_cnt);
   tag_out <= tag_out_lcl;
   
end architecture Behavioral;
