
-- Project: Exercises, 7-segment, 4 displays
--
-- !!!! this is for the Terasic board !!!!

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

entity seg_control is
  port 
   (
    clk          : in   std_logic;
    pb_debncd_1  : in   std_logic;
    pb_debncd_2  : in   std_logic;
    --
    seven_seg_1  : out  std_logic_vector(7 downto 0);
    seven_seg_2  : out  std_logic_vector(7 downto 0);
    seven_seg_3  : out  std_logic_vector(7 downto 0);
    seven_seg_4  : out  std_logic_vector(7 downto 0);
    en1          : out  std_logic;
    en2          : out  std_logic;
    en3          : out  std_logic;
    en4          : out  std_logic
   );
end entity;

architecture Behavioral of seg_control is

   signal pb_debncd_1_d1   : std_logic;
   signal pb_debncd_2_d1   : std_logic;
   signal select_cnt       : unsigned(1 downto 0);
   signal display_cnt_1    : unsigned(3 downto 0);
   signal display_cnt_2    : unsigned(3 downto 0);
   signal display_cnt_3    : unsigned(3 downto 0);
   signal display_cnt_4    : unsigned(3 downto 0);
   signal temp_display     : std_logic_vector(7 downto 0);
   signal temp_display_cnt : unsigned(3 downto 0);
   
begin

   en1 <= '0'; -- not used on the Terasic board
   en2 <= '0';
   en3 <= '0';
   en4 <= '0';
   
   seg_control_process : process(clk)
   begin
      if rising_edge(clk) then
         pb_debncd_1_d1 <= pb_debncd_1;
         pb_debncd_2_d1 <= pb_debncd_2;
         --
         if    (    pb_debncd_1    = '0' --Terasic PBs are low-active
                AND pb_debncd_1_d1 = '1'
               ) then 
               select_cnt  <= select_cnt + 1;
         elsif (    pb_debncd_2    = '0' --Terasic PBs are low-active
                AND pb_debncd_2_d1 = '1'
               ) then 
            case (select_cnt) is 
               when "00" =>  display_cnt_1 <= display_cnt_1 + 1;
               when "01" =>  display_cnt_2 <= display_cnt_2 + 1;
               when "10" =>  display_cnt_3 <= display_cnt_3 + 1;
               when "11" =>  display_cnt_4 <= display_cnt_4 + 1;
               when others => null;
            end case;
         end if;
      end if;
   end process;

   --      0
   --    5   1
   --      6
   --    4   2
   --      3
   
   seg_decode_process : process(clk)
   begin
      if rising_edge(clk) then
         seven_seg_1 <= "11111111";
         seven_seg_2 <= "11111111";
         seven_seg_3 <= "11111111";
         seven_seg_4 <= "11111111";
         --         
         case (select_cnt) is
            when "00" => temp_display_cnt <= display_cnt_1;
            when "01" => temp_display_cnt <= display_cnt_2;
            when "10" => temp_display_cnt <= display_cnt_3;
            when "11" => temp_display_cnt <= display_cnt_4;
            when others => null;
         end case;
         --
         case(temp_display_cnt) is       -- 76543210
            when X"0" => temp_display <= "11000000";
            when X"1" => temp_display <= "11111001";
            when X"2" => temp_display <= "10100100";
            when X"3" => temp_display <= "10110000";
            when X"4" => temp_display <= "10011001";
            when X"5" => temp_display <= "10010010";
            when X"6" => temp_display <= "10000010";
            when X"7" => temp_display <= "11111000";
            when X"8" => temp_display <= "10000000";
            when X"9" => temp_display <= "10010000";
            when X"A" => temp_display <= "10001000";
            when X"b" => temp_display <= "10000011";
            when X"C" => temp_display <= "11000110";
            when X"d" => temp_display <= "10100001";
            when X"E" => temp_display <= "10000110";
            when X"F" => temp_display <= "10001110";
            when others => null;
         end case;
         --
         case (select_cnt) is
            when "00" => seven_seg_1 <= temp_display;
            when "01" => seven_seg_2 <= temp_display;
            when "10" => seven_seg_3 <= temp_display;
            when "11" => seven_seg_4 <= temp_display;
            when others => null;
         end case;
      end if;
   end process;
         

      
--       if (select_cnt = "00) then
--          seven_seg_1 <= "11111111";
--       else -- display segments are low-active
--          case(display_cnt_1) is       -- 76543210
--             when X"0" => seven_seg_1 <= "11000000";
--             when X"1" => seven_seg_1 <= "11111001";
--             when X"2" => seven_seg_1 <= "10100100";
--             when X"3" => seven_seg_1 <= "10110000";
--             when X"4" => seven_seg_1 <= "10011001";
--             when X"5" => seven_seg_1 <= "10010010";
--             when X"6" => seven_seg_1 <= "10000010";
--             when X"7" => seven_seg_1 <= "11111000";
--             when X"8" => seven_seg_1 <= "10000000";
--             when X"9" => seven_seg_1 <= "10010000";
--             when X"A" => seven_seg_1 <= "10001000";
--             when X"b" => seven_seg_1 <= "10000011";
--             when X"C" => seven_seg_1 <= "11000110";
--             when X"d" => seven_seg_1 <= "10100001";
--             when X"E" => seven_seg_1 <= "10000110";
--             when X"F" => seven_seg_1 <= "10001110";
--             when others => seven_seg_1 <= "11111111";
--          end case;
--       end if;
--       --
--       if (display_select = '0') then
--          seven_seg_2 <= "11111111";
--       else
--          case(display_cnt_2) is       -- 76543210
--             when X"0" => seven_seg_2 <= "11000000";
--             when X"1" => seven_seg_2 <= "11111001";
--             when X"2" => seven_seg_2 <= "10100100";
--             when X"3" => seven_seg_2 <= "10110000";
--             when X"4" => seven_seg_2 <= "10011001";
--             when X"5" => seven_seg_2 <= "10010010";
--             when X"6" => seven_seg_2 <= "10000010";
--             when X"7" => seven_seg_2 <= "11111000";
--             when X"8" => seven_seg_2 <= "10000000";
--             when X"9" => seven_seg_2 <= "10010000";
--             when X"A" => seven_seg_2 <= "10001000";
--             when X"b" => seven_seg_2 <= "10000011";
--             when X"C" => seven_seg_2 <= "11000110";
--             when X"d" => seven_seg_2 <= "10100001";
--             when X"E" => seven_seg_2 <= "10000110";
--             when X"F" => seven_seg_2 <= "10001110";
--             when others => seven_seg_2 <= "11111111";
--          end case;
--       end if;
--    end if;
-- end process;
   
end architecture Behavioral;

