
-- Project: fifo_game
--
-- pseudo-random generation and timing.

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

use work.projects_pkg.all;

entity pr_gen is
  port 
   (
    clk          : in  std_logic;
    start        : in  std_logic;
    speed_select : in  std_logic;
    prbs         : out std_logic_vector(2 downto 0);
    tag          : out std_logic; --active for one full cycle time
    step         : out std_logic
   );
end entity;

architecture Behavioral of pr_gen is

  signal prs         : std_logic_vector(11 downto 1);
  signal step_lcl    : std_logic;
  signal tag_lcl     : std_logic;
  signal tag_lcl_d1  : std_logic;
  signal tag_lcl_d2  : std_logic;
  signal tag_lcl_d3  : std_logic;
  signal step_cnt    : unsigned(27 downto 0);
  signal load_cnt    : unsigned(10 downto 0);
  signal prs_prev    : std_logic_vector(3 downto 1);
  signal tag_check   : std_logic_vector(2 downto 0);
  signal prs_alt     : std_logic_vector(2 downto 0);
   
begin

  step_cnt_proc : process(clk)
  begin
     if rising_edge(clk) then
        if (step_cnt = X"0000000") then
           if (speed_select = '0') then
              step_cnt <= step_load_slow;
           else
              step_cnt <= step_load_fast;
           end if;
           --
           step_lcl <= '1';
        else
           step_cnt <= step_cnt - 1;
           step_lcl <= '0';
        end if;
     end if;
  end process;

  load_cnt_proc : process(clk)
  begin
     if rising_edge(clk) then
       load_cnt <= load_cnt + 1;
     end if;
  end process;

  prs_process : process(clk)
  begin
     if rising_edge(clk) then
        if (start = '1') then --load seeds
           prs <= std_logic_vector(load_cnt);
        elsif (step_lcl = '1') then  -- x^11 + x^9 + x^1
           prs      <=   prs(1) 
                       & prs(11) 
                       & (prs(10) xor prs(1)) 
                       & prs(9 downto 2);
           prs_prev <= prs(3 downto 1);
        end if;
        --
        if (step_lcl = '1') then
           if (    tag_check = "111" 
               AND tag_lcl    /= '1' --previous
               AND tag_lcl_d1 /= '1' --two prior
               AND tag_lcl_d2 /= '1' --three prior
               AND tag_lcl_d3 /= '1' --four prior
              ) then 
              tag_lcl <= '1';
           else
              tag_lcl <= '0';
           end if;
           --
           tag_lcl_d1 <= tag_lcl;
           tag_lcl_d2 <= tag_lcl_d1;
           tag_lcl_d3 <= tag_lcl_d2;
        end if;
     end if;
  end process;

  tag_check <= (prs(11) & prs(8) & prs(5));
  
  prs_alt <= (prs(9) & prs(5) & prs(2));
  prbs    <= prs(3 downto 1) when (prs(3 downto 1) /= prs_prev)
                          else
          prs(3 downto 1) XOR prs_alt;
  --
  step <= step_lcl;
  tag  <= tag_lcl;
  
end architecture Behavioral;

