
-- Project: general
--
-- translates a 4-bit hex value to an 8-bit 7-segment drive signal.

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

entity seg_encode is
  port 
   (
--  hex_in     : in   std_logic_vector(3 downto 0);
    hex_in     : in   std_logic_vector(4 downto 0);
    seven_seg  : out  std_logic_vector(7 downto 0)
   );
end entity;

architecture Behavioral of seg_encode is
   
begin

   --      0
   --    5   1
   --      6
   --    4   2
   --      3
   
   seg_decode_process : process(hex_in)
   begin
     case(hex_in(3 downto 0)) is                       -- 76543210
        when X"0" => seven_seg(6 downto 0) <= "1000000";
        when X"1" => seven_seg(6 downto 0) <= "1111001";
        when X"2" => seven_seg(6 downto 0) <= "0100100";
        when X"3" => seven_seg(6 downto 0) <= "0110000";
        when X"4" => seven_seg(6 downto 0) <= "0011001";
        when X"5" => seven_seg(6 downto 0) <= "0010010";
        when X"6" => seven_seg(6 downto 0) <= "0000010";
        when X"7" => seven_seg(6 downto 0) <= "1111000";
        when X"8" => seven_seg(6 downto 0) <= "0000000";
        when X"9" => seven_seg(6 downto 0) <= "0010000";
        when X"A" => seven_seg(6 downto 0) <= "0001000";
        when X"b" => seven_seg(6 downto 0) <= "0000011";
        when X"C" => seven_seg(6 downto 0) <= "1000110";
        when X"d" => seven_seg(6 downto 0) <= "0100001";
        when X"E" => seven_seg(6 downto 0) <= "0000110";
        when X"F" => seven_seg(6 downto 0) <= "0001110";
        when others => seven_seg(6 downto 0) <= "1111111";
     end case;
   end process;

   seven_seg(7) <= not hex_in(4); --low-active


   
end architecture Behavioral;
