
-- Project: memories
--
-- Controls two external 7-segment displays.
--
-- !!!! this is for the Digilent board !!!!

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

entity seg_drive is
  port 
   (
    clk          : in   std_logic;
    mem_dat_1    : in   std_logic_vector(7 downto 0);
    mem_dat_2    : in   std_logic_vector(7 downto 0);
    --
    seven_seg_1  : out  std_logic_vector(7 downto 0);
    seven_seg_2  : out  std_logic_vector(7 downto 0);
    seven_seg_3  : out  std_logic_vector(7 downto 0);
    seven_seg_4  : out  std_logic_vector(7 downto 0);
    en1          : out  std_logic;
    en2          : out  std_logic;
    en3          : out  std_logic;
    en4          : out  std_logic 
   );
end entity;

architecture Behavioral of seg_drive is

  component seg_encode
  port 
   (
    hex_in     : in   std_logic_vector(3 downto 0);
    seven_seg  : out  std_logic_vector(7 downto 0)
   );
  end component;
  
  signal ms_cnt       : unsigned(16 downto 0);
  signal seg_sel      : unsigned(1 downto 0);
  signal seg_1        : std_logic_vector(7 downto 0);
  signal seg_2        : std_logic_vector(7 downto 0);
  signal seg_3        : std_logic_vector(7 downto 0);
  signal seg_4        : std_logic_vector(7 downto 0);
   
begin

  seg_encode_1 : seg_encode
  port map
   (
    hex_in     => mem_dat_1(7 downto 4), 
    seven_seg  => seg_1
   );

  seg_encode_2 : seg_encode
  port map
   (
    hex_in     => mem_dat_1(3 downto 0),
    seven_seg  => seg_2
   );
   
  seg_encode_3 : seg_encode
  port map
   (
    hex_in     => mem_dat_2(7 downto 4), 
    seven_seg  => seg_3
   );

  seg_encode_4 : seg_encode
  port map
   (
    hex_in     => mem_dat_2(3 downto 0),
    seven_seg  => seg_4
   );
   
  -- Cycle through the four segments.
  main_proc : process(clk)
  begin
    if rising_edge(clk) then
      ms_cnt <= ms_cnt + 1; -- ~~1.3ms at 100MHz
      --
      if (ms_cnt = '0' & X"0000") then
        seg_sel <= seg_sel + 1;
      end if;
    end if;
  end process;

  -- 7-segment enables are low-active
  en1 <= '0' when seg_sel = "00" else '1';
  en2 <= '0' when seg_sel = "01" else '1';
  en3 <= '0' when seg_sel = "10" else '1';
  en4 <= '0' when seg_sel = "11" else '1';

  seven_seg_1 <= seg_1 when seg_sel = "00" else 
                 seg_2 when seg_sel = "01" else 
                 seg_3 when seg_sel = "10" else 
                 seg_4;

  seven_seg_2 <= X"00";
  seven_seg_3 <= X"00";
  seven_seg_4 <= X"00";
                 
end architecture Behavioral;

